@@ -28,9 +28,11 @@
 
 #include <string.h>
 
+#include <libpeas/peas-engine.h>
 #include <libpeas/peas-i18n.h>
 
 #include "peas-gtk-plugin-manager-view.h"
+#include "peas-gtk-disable-plugins-dialog.h"
 #include "peas-gtk-plugin-manager-store.h"
 #include "peas-gtk-configurable.h"
 
@@ -118,6 +120,73 @@ convert_child_iter_to_iter (PeasGtkPluginManagerView *view,
   return success;
 }
 
+static GList *
+get_dependant_plugins (PeasGtkPluginManagerView *view,
+                       PeasPluginInfo           *info)
+{
+  const gchar *module_name;
+  const GList *plugins;
+  GList *dep_plugins = NULL;
+
+  module_name = peas_plugin_info_get_module_name (info);
+  plugins = peas_engine_get_plugin_list (view->priv->engine);
+
+  for (; plugins != NULL; plugins = plugins->next)
+    {
+      PeasPluginInfo *plugin = (PeasPluginInfo *) plugins->data;
+
+      if (!peas_plugin_info_is_loaded (plugin))
+        continue;
+
+      /* Don't add builtin plugins if they are not shown */
+      if (!view->priv->show_builtin && peas_plugin_info_is_builtin (plugin))
+        continue;
+
+      if (peas_plugin_info_has_dependency (plugin, module_name))
+        dep_plugins = g_list_prepend (dep_plugins, plugin);
+    }
+
+  return dep_plugins;
+}
+
+static void
+toggle_enabled (PeasGtkPluginManagerView *view,
+                GtkTreeIter              *iter)
+{
+  PeasPluginInfo *info;
+
+  info = peas_gtk_plugin_manager_store_get_plugin (view->priv->store, iter);
+
+  if (peas_plugin_info_is_loaded (info))
+    {
+      GList *dep_plugins;
+
+      dep_plugins = get_dependant_plugins (view, info);
+
+      if (dep_plugins != NULL)
+        {
+          GtkWindow *parent;
+          GtkWidget *dialog;
+          gint response;
+
+          parent = GTK_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (view)));
+
+          /* The dialog takes the list so don't free it */
+          dialog = peas_gtk_disable_plugins_dialog_new (parent, info,
+                                                        dep_plugins);
+
+          response = gtk_dialog_run (GTK_DIALOG (dialog));
+
+          gtk_widget_destroy (dialog);
+
+          if (response != GTK_RESPONSE_OK)
+            return;
+        }
+    }
+
+  peas_gtk_plugin_manager_store_toggle_enabled (view->priv->store, iter);
+}
+
 static void
 plugin_list_changed_cb (PeasEngine               *engine,
                         GParamSpec               *pspec,
@@ -129,24 +198,6 @@ plugin_list_changed_cb (PeasEngine               *engine,
 
   peas_gtk_plugin_manager_store_reload (view->priv->store);
 
-  if (info == NULL)
-    {
-      GtkTreeModel *model;
-      GtkTreeIter iter;
-
-      model = gtk_tree_view_get_model (GTK_TREE_VIEW (view));
-
-      if (gtk_tree_model_get_iter_first (model, &iter))
-        {
-          PeasGtkPluginManagerStore *store;
-
-          store = PEAS_GTK_PLUGIN_MANAGER_STORE (view->priv->store);
-
-          convert_iter_to_child_iter (view, &iter);
-          info = peas_gtk_plugin_manager_store_get_plugin (store, &iter);
-        }
-    }
-
   if (info != NULL)
     peas_gtk_plugin_manager_view_set_selected_plugin (view, info);
 }
@@ -184,7 +235,7 @@ enabled_toggled_cb (GtkCellRendererToggle    *cell,
   if (gtk_tree_model_get_iter (model, &iter, path))
     {
       convert_iter_to_child_iter (view, &iter);
-      peas_gtk_plugin_manager_store_toggle_enabled (view->priv->store, &iter);
+      toggle_enabled (view, &iter);
     }
 
   gtk_tree_path_free (path);
@@ -198,6 +249,7 @@ name_search_cb (GtkTreeModel             *model,
                 GtkTreeIter              *iter,
                 PeasGtkPluginManagerView *view)
 {
+  GtkTreeIter child_iter = *iter;
   PeasPluginInfo *info;
   gchar *normalized_string;
   gchar *normalized_key;
@@ -206,7 +258,8 @@ name_search_cb (GtkTreeModel             *model,
   gint key_len;
   gboolean retval;
 
-  info = peas_gtk_plugin_manager_store_get_plugin (view->priv->store, iter);
+  convert_iter_to_child_iter (view, &child_iter);
+  info = peas_gtk_plugin_manager_store_get_plugin (view->priv->store, &child_iter);
 
   if (info == NULL)
     return FALSE;
@@ -244,7 +297,7 @@ enabled_menu_cb (GtkMenu                  *menu,
 
   convert_iter_to_child_iter (view, &iter);
 
-  peas_gtk_plugin_manager_store_toggle_enabled (view->priv->store, &iter);
+  toggle_enabled (view, &iter);
 }
 
 static void
@@ -380,7 +433,7 @@ menu_position_under_tree_view (GtkMenu     *menu,
   *push_in = TRUE;
 }
 
-static void
+static gboolean
 show_popup_menu (GtkTreeView              *tree_view,
                  PeasGtkPluginManagerView *view,
                  GdkEventButton           *event)
@@ -391,7 +444,7 @@ show_popup_menu (GtkTreeView              *tree_view,
   view->priv->popup_menu = create_popup_menu (view);
 
   if (view->priv->popup_menu == NULL)
-    return;
+    return FALSE;
 
   gtk_menu_attach_to_widget (GTK_MENU (view->priv->popup_menu),
                              GTK_WIDGET (view),
@@ -411,6 +464,8 @@ show_popup_menu (GtkTreeView              *tree_view,
       gtk_menu_shell_select_first (GTK_MENU_SHELL (view->priv->popup_menu),
                                    FALSE);
     }
+
+  return TRUE;
 }
 
 static void
@@ -520,19 +575,15 @@ peas_gtk_plugin_manager_view_button_press_event (GtkWidget      *tree_view,
   if (event->type != GDK_BUTTON_PRESS || event->button != 3 || !handled)
     return handled;
 
-  show_popup_menu (GTK_TREE_VIEW (tree_view), view, event);
-
-  return TRUE;
+  return show_popup_menu (GTK_TREE_VIEW (tree_view), view, event);
 }
 
 static gboolean
-peas_gtk_plugin_manager_view_popup_menu (GtkWidget *widget)
+peas_gtk_plugin_manager_view_popup_menu (GtkWidget *tree_view)
 {
-  GtkTreeView *tree_view = GTK_TREE_VIEW (widget);
+  PeasGtkPluginManagerView *view = PEAS_GTK_PLUGIN_MANAGER_VIEW (tree_view);
 
-  show_popup_menu (tree_view, PEAS_GTK_PLUGIN_MANAGER_VIEW (tree_view), NULL);
-
-  return TRUE;
+  return show_popup_menu (GTK_TREE_VIEW (tree_view), view, NULL);
 }
 
 static gboolean
@@ -594,7 +645,7 @@ peas_gtk_plugin_manager_view_row_activated (GtkTreeView       *tree_view,
   convert_iter_to_child_iter (view, &iter);
 
   if (peas_gtk_plugin_manager_store_can_enable (view->priv->store, &iter))
-    peas_gtk_plugin_manager_store_toggle_enabled (view->priv->store, &iter);
+    toggle_enabled (view, &iter);
 }
 
 static void
