@@ -69,6 +69,13 @@ struct _PeasGtkPluginManagerPrivate {
   GtkWidget *configure_button;
 };
 
+/* Properties */
+enum {
+  PROP_0,
+  PROP_ENGINE,
+  PROP_VIEW
+};
+
 G_DEFINE_TYPE (PeasGtkPluginManager, peas_gtk_plugin_manager, GTK_TYPE_VBOX);
 
 static gboolean
@@ -356,24 +363,16 @@ peas_gtk_plugin_manager_init (PeasGtkPluginManager *pm)
 {
   GtkWidget *label;
   GtkWidget *hbuttonbox;
-  GtkTreeSelection *selection;
 
   pm->priv = G_TYPE_INSTANCE_GET_PRIVATE (pm,
                                           PEAS_GTK_TYPE_PLUGIN_MANAGER,
                                           PeasGtkPluginManagerPrivate);
 
-  /**
-   * If we are using a PeasGtkPluginManager, we know for sure we will be using
-   * libpeas-gtk, so let's load the typelib for it here.
-   */
+  /* If we are using a PeasGtkPluginManager, we know for sure we will be using
+     libpeas-gtk, so let's load the typelib for it here. */
   g_irepository_require (g_irepository_get_default (),
                          "PeasGtk", "1.0", 0, NULL);
 
-  /**
-   * Let's initialize the widgets of the plugin manager now.
-   */
-  pm->priv->engine = g_object_ref (peas_engine_get_default ());
-
   gtk_box_set_spacing (GTK_BOX (pm), 6);
 
   gtk_widget_push_composite_child ();
@@ -391,9 +390,6 @@ peas_gtk_plugin_manager_init (PeasGtkPluginManager *pm)
   gtk_label_set_mnemonic_widget (GTK_LABEL (label), pm->priv->sw);
   gtk_box_pack_start (GTK_BOX (pm), pm->priv->sw, TRUE, TRUE, 0);
 
-  pm->priv->view = peas_gtk_plugin_manager_view_new ();
-  gtk_container_add (GTK_CONTAINER (pm->priv->sw), pm->priv->view);
-
   hbuttonbox = gtk_hbutton_box_new ();
   gtk_button_box_set_layout (GTK_BUTTON_BOX (hbuttonbox), GTK_BUTTONBOX_END);
   gtk_box_set_spacing (GTK_BOX (hbuttonbox), 6);
@@ -418,22 +414,99 @@ peas_gtk_plugin_manager_init (PeasGtkPluginManager *pm)
                     "clicked",
                     G_CALLBACK (show_configure_cb),
                     pm);
+}
 
+static void
+peas_gtk_plugin_manager_set_property (GObject      *object,
+                                      guint         prop_id,
+                                      const GValue *value,
+                                      GParamSpec   *pspec)
+{
+  PeasGtkPluginManager *pm = PEAS_GTK_PLUGIN_MANAGER (object);
+
+  switch (prop_id)
+    {
+    case PROP_ENGINE:
+      pm->priv->engine = g_value_get_object (value);
+      break;
+    case PROP_VIEW:
+      pm->priv->view = g_value_get_object (value);
+      break;
+    default:
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+      break;
+    }
+}
+
+static void
+peas_gtk_plugin_manager_get_property (GObject    *object,
+                                      guint       prop_id,
+                                      GValue     *value,
+                                      GParamSpec *pspec)
+{
+  PeasGtkPluginManager *pm = PEAS_GTK_PLUGIN_MANAGER (object);
+
+  switch (prop_id)
+    {
+    case PROP_ENGINE:
+      g_value_set_object (value, pm->priv->engine);
+      break;
+    case PROP_VIEW:
+      g_value_set_object (value, peas_gtk_plugin_manager_get_view (pm));
+      break;
+    default:
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+      break;
+    }
+}
+
+static void
+peas_gtk_plugin_manager_constructed (GObject *object)
+{
+  PeasGtkPluginManager *pm = PEAS_GTK_PLUGIN_MANAGER (object);
+  GtkTreeSelection *selection;
+
+  if (pm->priv->engine == NULL)
+    pm->priv->engine = peas_engine_get_default ();
+
+  g_object_ref (pm->priv->engine);
 
   /* When we create the manager, we always rescan the plugins directory
-     Must come after the view has connected to notify::plugin-list */
+   * Must come after the view has connected to notify::plugin-list
+   */
   peas_engine_rescan_plugins (pm->priv->engine);
 
+  /* For the view to behave as expected, we must ensure it uses the same
+   * engine as the manager itself.
+   */
+  if (pm->priv->view != NULL)
+    {
+      PeasEngine *engine;
+
+      g_object_get (pm->priv->view,
+                    "engine", &engine,
+                    NULL);
+
+      g_warn_if_fail (engine == pm->priv->engine);
+
+      if (engine != pm->priv->engine)
+        {
+          g_object_unref (pm->priv->view);
+          pm->priv->view = NULL;
+        }
+
+      g_object_unref (engine);
+    }
+
+  if (pm->priv->view == NULL)
+    pm->priv->view = peas_gtk_plugin_manager_view_new (pm->priv->engine);
+
+  gtk_widget_push_composite_child ();
+  gtk_container_add (GTK_CONTAINER (pm->priv->sw), pm->priv->view);
+  gtk_widget_pop_composite_child ();
+
   selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (pm->priv->view));
 
-  g_signal_connect_after (pm->priv->engine,
-                          "load-plugin",
-                          G_CALLBACK (plugin_loaded_toggled_cb),
-                          pm);
-  g_signal_connect_after (pm->priv->engine,
-                          "unload-plugin",
-                          G_CALLBACK (plugin_loaded_toggled_cb),
-                          pm);
   g_signal_connect_swapped (selection,
                             "changed",
                             G_CALLBACK (selection_changed_cb),
@@ -446,9 +519,20 @@ peas_gtk_plugin_manager_init (PeasGtkPluginManager *pm)
                     "populate-popup",
                     G_CALLBACK (populate_popup_cb),
                     pm);
+  g_signal_connect_after (pm->priv->engine,
+                          "load-plugin",
+                          G_CALLBACK (plugin_loaded_toggled_cb),
+                          pm);
+  g_signal_connect_after (pm->priv->engine,
+                          "unload-plugin",
+                          G_CALLBACK (plugin_loaded_toggled_cb),
+                          pm);
 
   /* Update the button sensitivity */
   selection_changed_cb (pm);
+
+  if (G_OBJECT_CLASS (peas_gtk_plugin_manager_parent_class)->constructed != NULL)
+    G_OBJECT_CLASS (peas_gtk_plugin_manager_parent_class)->constructed (object);
 }
 
 static void
@@ -474,22 +558,61 @@ peas_gtk_plugin_manager_class_init (PeasGtkPluginManagerClass *klass)
 {
   GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
+  object_class->set_property = peas_gtk_plugin_manager_set_property;
+  object_class->get_property = peas_gtk_plugin_manager_get_property;
+  object_class->constructed = peas_gtk_plugin_manager_constructed;
   object_class->dispose = peas_gtk_plugin_manager_dispose;
 
+  /**
+   * PeasGtkPluginManager:engine:
+   *
+   * The #PeasEngine this manager is attached to.
+   */
+  g_object_class_install_property (object_class,
+                                   PROP_ENGINE,
+                                   g_param_spec_object ("engine",
+                                                        "engine",
+                                                        "The PeasEngine this manager is attached to",
+                                                        PEAS_TYPE_ENGINE,
+                                                        G_PARAM_READWRITE |
+                                                        G_PARAM_CONSTRUCT_ONLY |
+                                                        G_PARAM_STATIC_STRINGS));
+
+  /**
+   * PeasGtkPluginManager:view:
+   *
+   * The #PeasGtkPluginManagerView shown in the #PeasGtkPluginManager.
+   */
+  g_object_class_install_property (object_class,
+                                   PROP_VIEW,
+                                   g_param_spec_object ("view",
+                                                        "view",
+                                                        "The view shown in the manager",
+                                                        PEAS_GTK_TYPE_PLUGIN_MANAGER_VIEW,
+                                                        G_PARAM_READWRITE |
+                                                        G_PARAM_CONSTRUCT_ONLY |
+                                                        G_PARAM_STATIC_STRINGS));
+
   g_type_class_add_private (object_class, sizeof (PeasGtkPluginManagerPrivate));
 }
 
 /**
  * peas_gtk_plugin_manager_new:
+ * @engine: (allow-none): A #PeasEngine, or %NULL.
  *
- * Creates a new plugin manager.
+ * Creates a new plugin manager for the given #PeasEngine.
+ *
+ * If @engine is %NULL, then the default engine will be used.
  *
  * Returns: the new #PeasGtkPluginManager.
  */
 GtkWidget *
-peas_gtk_plugin_manager_new (void)
+peas_gtk_plugin_manager_new (PeasEngine *engine)
 {
+  g_return_val_if_fail (engine == NULL || PEAS_IS_ENGINE (engine), NULL);
+
   return GTK_WIDGET (g_object_new (PEAS_GTK_TYPE_PLUGIN_MANAGER,
+                                   "engine", engine,
                                    NULL));
 }
 
