@@ -1,5 +1,5 @@
 /*
- * peas-plugin.h
+ * peas-extension-base.h
  * This file is part of libpeas
  *
  * Copyright (C) 2002-2005 - Paolo Maggi
@@ -26,9 +26,6 @@
 
 #include "peas-plugin-info.h"
 
-/* TODO: add a .h file that includes all the .h files normally needed to
- * develop a plugin */
-
 G_BEGIN_DECLS
 
 /*
@@ -57,6 +54,8 @@ struct _PeasExtensionBase {
 
 struct _PeasExtensionBaseClass {
   GObjectClass parent_class;
+
+  gpointer padding[8];
 };
 
 /*
