@@ -32,7 +32,6 @@ G_DEFINE_TYPE (PeasExtensionSeed, peas_extension_seed, PEAS_TYPE_EXTENSION);
 
 enum {
   PROP_0,
-  PROP_EXTEN_TYPE,
   PROP_JS_CONTEXT,
   PROP_JS_OBJECT,
 };
@@ -80,12 +79,18 @@ peas_extension_seed_constructed (GObject *object)
 }
 
 static void
-peas_extension_seed_finalize (GObject *object)
+peas_extension_seed_dispose (GObject *object)
 {
   PeasExtensionSeed *sexten = PEAS_EXTENSION_SEED (object);
 
-  seed_value_unprotect (sexten->js_context, sexten->js_object);
-  seed_context_unref (sexten->js_context);
+  if (sexten->js_object != NULL)
+    {
+      seed_value_unprotect (sexten->js_context, sexten->js_object);
+      seed_context_unref (sexten->js_context);
+
+      sexten->js_object = NULL;
+      sexten->js_context = NULL;
+    }
 }
 
 static SeedValue
@@ -226,16 +231,17 @@ set_return_value (OutArg        *arg,
 
 static gboolean
 peas_extension_seed_call (PeasExtension *exten,
+                          GType          exten_type,
                           const gchar   *method_name,
                           GIArgument    *args,
                           GIArgument    *retval)
 {
   PeasExtensionSeed *sexten = PEAS_EXTENSION_SEED (exten);
-  GType exten_type;
   SeedValue js_method;
   GICallableInfo *func_info;
   GITypeInfo *retval_info;
-  guint n_args, n_in_args, n_out_args, i;
+  gint n_args;
+  guint n_in_args, n_out_args, i;
   SeedValue *js_in_args;
   OutArg *out_args;
   SeedValue js_ret, val;
@@ -245,7 +251,8 @@ peas_extension_seed_call (PeasExtension *exten,
   g_return_val_if_fail (sexten->js_context != NULL, FALSE);
   g_return_val_if_fail (sexten->js_object != NULL, FALSE);
 
-  exten_type = peas_extension_get_extension_type (exten);
+  if (exten_type == G_TYPE_INVALID)
+    exten_type = peas_extension_get_extension_type (exten);
 
   /* Fetch the JS method we want to call */
   js_method = seed_object_get_property (sexten->js_context,
@@ -268,7 +275,11 @@ peas_extension_seed_call (PeasExtension *exten,
 
   /* Prepare the arguments */
   func_info = peas_gi_get_method_info (exten_type, method_name);
+  if (func_info == NULL)
+    return FALSE;
+
   n_args = g_callable_info_get_n_args (func_info);
+  g_return_val_if_fail (n_args >= 0, FALSE);
   n_in_args = 0;
   n_out_args = 0;
 
@@ -290,13 +301,13 @@ peas_extension_seed_call (PeasExtension *exten,
   /* Handle the other arguments */
   for (i = 0; i < n_args && exc == NULL; i++)
     {
-      GIArgInfo *arg_info;
+      GIArgInfo arg_info;
       GITypeInfo *arg_type_info;
 
-      arg_info = g_callable_info_get_arg (func_info, i);
-      arg_type_info = g_arg_info_get_type (arg_info);
+      g_callable_info_load_arg (func_info, i, &arg_info);
+      arg_type_info = g_arg_info_get_type (&arg_info);
 
-      switch (g_arg_info_get_direction (arg_info))
+      switch (g_arg_info_get_direction (&arg_info))
         {
         case GI_DIRECTION_IN:
           js_in_args[n_in_args++] = get_argument (sexten->js_context,
@@ -319,7 +330,6 @@ peas_extension_seed_call (PeasExtension *exten,
         }
 
       g_base_info_unref ((GIBaseInfo *) arg_type_info);
-      g_base_info_unref ((GIBaseInfo *) arg_info);
     }
   if (exc != NULL)
     goto cleanup;
@@ -374,7 +384,7 @@ peas_extension_seed_class_init (PeasExtensionSeedClass *klass)
 
   object_class->set_property = peas_extension_seed_set_property;
   object_class->constructed = peas_extension_seed_constructed;
-  object_class->finalize = peas_extension_seed_finalize;
+  object_class->dispose = peas_extension_seed_dispose;
 
   extension_class->call = peas_extension_seed_call;
 
