@@ -43,6 +43,9 @@ void testing_extension_test_teardown_ (TestingExtensionFixture_ *fixture,
 void testing_extension_test_runner_   (TestingExtensionFixture_ *fixture,
                                        gconstpointer             data);
 
+void testing_extension_garbage_collect_           (PeasEngine *engine);
+void testing_extension_provides_valid_            (PeasEngine *engine);
+void testing_extension_provides_invalid_          (PeasEngine *engine);
 void testing_extension_create_valid_              (PeasEngine *engine);
 void testing_extension_create_invalid_            (PeasEngine *engine);
 void testing_extension_reload_                    (PeasEngine *engine);
@@ -56,6 +59,11 @@ void testing_extension_properties_read_only_      (PeasEngine *engine);
 void testing_extension_properties_write_only_     (PeasEngine *engine);
 void testing_extension_properties_readwrite_      (PeasEngine *engine);
 
+#define _EXTENSION_TESTS_INIT(loader) \
+  testing_init (); \
+  peas_engine_enable_loader (peas_engine_get_default (), loader); \
+  g_object_unref (peas_engine_get_default ()); \
+  testing_extension_set_plugin_ ("extension-" loader);
 
 #define _EXTENSION_TEST(loader, path, ftest) \
   g_test_add ("/extension/" loader "/" path, TestingExtensionFixture_, \
@@ -65,19 +73,12 @@ void testing_extension_properties_readwrite_      (PeasEngine *engine);
               testing_extension_test_teardown_);
 
 #define EXTENSION_TESTS(loader) \
-int \
-main (int   argc, \
-      char *argv[]) \
-{ \
-  g_test_init (&argc, &argv, NULL); \
+  _EXTENSION_TESTS_INIT(loader); \
 \
-  g_type_init (); \
+  _EXTENSION_TEST (loader, "garbage-collect", garbage_collect); \
 \
-  testing_init (); \
-\
-  peas_engine_enable_loader (peas_engine_get_default (), loader); \
-\
-  testing_extension_set_plugin_ ("extension-" loader); \
+  _EXTENSION_TEST (loader, "provides-valid", provides_valid); \
+  _EXTENSION_TEST (loader, "provides-invalid", provides_invalid); \
 \
   _EXTENSION_TEST (loader, "create-valid", create_valid); \
   _EXTENSION_TEST (loader, "create-invalid", create_invalid); \
@@ -93,10 +94,15 @@ main (int   argc, \
   _EXTENSION_TEST (loader, "properties-construct-only", properties_construct_only); \
   _EXTENSION_TEST (loader, "properties-read-only", properties_read_only); \
   _EXTENSION_TEST (loader, "properties-write-only", properties_write_only); \
-  _EXTENSION_TEST (loader, "properties-readwrite", properties_readwrite); \
-\
-  return g_test_run (); \
-}
+  _EXTENSION_TEST (loader, "properties-readwrite", properties_readwrite);
+
+/* This macro is there to add loader-specific tests. */
+#define EXTENSION_TEST_ADD(loader, path, func) \
+  g_test_add ("/extension/" loader "/" path, TestingExtensionFixture_, \
+              (gpointer) func, \
+              testing_extension_test_setup_, \
+              testing_extension_test_runner_, \
+              testing_extension_test_teardown_);
 
 G_END_DECLS
 
